# -*- coding: utf-8 -*-
"""
   Copyright 2015 Michael Schmuker
   
   Licensed under the Apache License, Version 2.0 (the "License");
   you may not use this file except in compliance with the License.
   You may obtain a copy of the License at

       http://www.apache.org/licenses/LICENSE-2.0

   Unless required by applicable law or agreed to in writing, software
   distributed under the License is distributed on an "AS IS" BASIS,
   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
   See the License for the specific language governing permissions and
   limitations under the License.

Utilities to process the two_sources... dataset.


Created on Mon Apr 27 17:39:00 2015

@author: micha
"""

import scipy.interpolate
import pandas as pd
import numpy as np


def resample(data, dt=0.1):
    """
    Resample the signal to sample rate parameter (like it should be).

    Step I  : Create new time series (uniform sample rate).
    Step II : Interpolate old signal (Univariate Spline through all points).
    Step III: Create new data frame with the interpolated time series.

    Keyword arguments:
    data - DataFrame with time as index.
    dt -- Target sample intervall in s

    """
    max_time = data.index.max()
    min_time = data.index.min()
    start = min_time + np.mod(min_time, dt)

    uniform_time = np.arange(start, max_time, dt)
    df_interpol = pd.DataFrame(index=uniform_time)
    df_interpol.index.name = "time"
    for cn in data.columns:
        f = scipy.interpolate.UnivariateSpline(
            x=data.index.get_values(), y=data[cn].get_values(), s=0, k=1
        )
        interpol_col = f(uniform_time)
        df_interpol[cn] = interpol_col
    return df_interpol
